define([
  "TerraMA2WebApp/geo/directives",
  "TerraMA2WebApp/geo/services"
], function(directivesApp, servicesApp) {

'use strict';

var moduleName = "terrama2.components.geo";

angular.module(moduleName, [directivesApp, servicesApp])
  .constant("Geometry", {
    "POLYGON": "Polygon",
    "POINT": "Point"
  })

  .constant("GeometryFormat", {
    "WKT": "WKT",
    "GEOJSON": "GEOJSON"
  })

  .constant("GeoJSON", {
    "type": "",
    "coordinates": [],
    "crs": {
      type: 'name',
      properties : {
        name: "EPSG:"
      }
    }
  })

  .factory("Polygon", ["GeoJSON", "Geometry", "GeometryFormat", function(GeoJSON, Geometry, GeometryFormat) {
    return {
      /**
       * @param {GeoJSON} geojson - A geojson object structure
       * @param {GeometryFormat} format - A javascript object with limits value. Default - GEOJSON
       * @todo implement WKT format
       * @return {Object} a javascript object representation
       */
      read: function(geojson, format) {
        var model = {};
        if (format === undefined || !format) {
          format = GeometryFormat.GEOJSON;
        }

        switch(format) {
          case GeometryFormat.GEOJSON:
            var coordinates = geojson.coordinates[0];
            model.minX = coordinates[0][0];
            model.minY = coordinates[0][1];
            model.maxX = coordinates[2][0];
            model.maxY = coordinates[2][1];
            model.srid = geojson.srid || 4326;
            break;
          case GeometryFormat.WKT: // TODO: implement it
            break;
        }
        return model;
      },
      /**
       * @param {Object} model - A javascript object with limits value.
       * @return {Object} a geojson object representation
       */
      build: function(model) {
        var geojson = angular.copy(GeoJSON, {});
        var coordinates = [
          [model.minX, model.minY],
          [model.minX, model.maxY],
          [model.maxX, model.maxY],
          [model.maxX, model.minY],
          [model.minX, model.minY]
        ];
        geojson.type = Geometry.POLYGON;
        geojson.coordinates.push(coordinates);
        geojson.crs.properties.name += model.srid;
        return geojson;
      }
    };
  }])

  .factory("Point", ["Geometry", "GeoJSON", function(Geometry, GeoJSON) {
    return {
      /**
       * @param {GeoJSON} geojson - A geojson object structure
       * @param {GeometryFormat} format - A javascript object with limits value. Default - GEOJSON
       * @todo implement WKT format
       * @return {Object} a javascript object representation
       */
      read: function(geojson, format) {
        var model = {};
        if (format === undefined || !format) {
          format = GeometryFormat.GEOJSON;
        }

        switch(format) {
          case GeometryFormat.GEOJSON:
            var coordinates = geojson.coordinates[0];
            model.x = coordinates[0][0];
            model.y = coordinates[0][1];
            break;
          case GeometryFormat.WKT: // TODO: implement it
            break;
        }
        return model;
      },
      /**
       * @param {Object} model - A javascript object with limits value. {x: 1, y: 2, srid: 4326}
       * @return {Object} a geojson object representation
       */
      build: function(model) {
        var geojson = angular.copy(GeoJSON, {});
        geojson.type = Geometry.POINT;
        geojson.coordinates.push(model.x);
        geojson.coordinates.push(model.y);
        geojson.crs.properties.name += model.srid;
        return geojson;
      }
    };
  }])

  .factory("GeoJsonBuilder", ["Geometry", "Polygon", "Point", function(Geometry, Polygon, Point) {
    /**
     * @param {Geometry} geometry - A const geometry enum
     * @param {Object} model - A javascript object with limits value
     * @return {Object} a geojson object
     */
    return function(geometry, model) {
      switch(geometry) {
        case Geometry.POLYGON:
          return Polygon.build(model);
        case Geometry.POINT:
          return Point.build(model);
        default:
          return {};
      }
    };
  }])
  
  /**
   * This factory injects all GeoLibraries available in order make it useful when handle multiple geo data types.
   * 
   * @param {Geometry} Geometry - TerraMA² Geometry Enum
   * @param {Polygon} Polygon - TerraMA² Polygon builder
   * @param {Point} Point - TerraMA² Point builder
   * @param {GeoJsonBuilder} GeoJsonBuilder - TerraMA² geojson builder
   * @returns {Object}
   */
  .factory("GeoLibs", ["Geometry", "Polygon", "Point", "GeoJsonBuilder", function(Geometry, Polygon, Point, GeoJsonBuilder) {
    return {
      geometry: Geometry,
      point: Point,
      polygon: Polygon,
      geojson: GeoJsonBuilder
    };
  }]);

  return moduleName;
});